/**
 * @file
 * JavaScript for the Realtime Text Preview module with CKEditor 5 support and similarity search.
 */

(function ($, Drupal, once, drupalSettings) {
  'use strict';

  // Debounce function to limit API calls
  function debounce(func, wait) {
    let timeout;
    return function executedFunction(...args) {
      const later = () => {
        clearTimeout(timeout);
        func(...args);
      };
      clearTimeout(timeout);
      timeout = setTimeout(later, wait);
    };
  }

  /**
   * Performs similarity search via AJAX.
   */
  function performSimilaritySearch(text, $similarityContainer) {
    // Show loading state
    $similarityContainer.html('<p class="similarity-loading">Searching for similar content...</p>');
    
    $.ajax({
      url: '/realtime-text-preview/similarity-search',
      type: 'POST',
      data: {
        text: text,
        limit: 5
      },
      success: function(response) {
        displaySimilarityResults(response, $similarityContainer);
      },
      error: function(xhr, status, error) {
        $similarityContainer.html('<p class="similarity-error">Error searching for similar content.</p>');
        console.error('Similarity search error:', error);
      }
    });
  }

  /**
   * Displays similarity search results.
   */
  function displaySimilarityResults(response, $container) {
    if (response.results && response.results.length > 0) {
      let html = '<div class="similarity-results"><h4>Similar Content Found:</h4><ul>';
      
      response.results.forEach(function(result) {
        html += '<li class="similarity-result-item">';
        html += '<a href="' + result.url + '" target="_blank">' + result.title + '</a>';
        html += ' <span class="similarity-score">(' + result.similarity + '% similar)</span>';
        html += '</li>';
      });
      
      html += '</ul></div>';
      $container.html(html);
    } else {
      $container.html('<p class="similarity-no-results">' + (response.message || 'No similar content found.') + '</p>');
    }
  }

  /**
   * Creates similarity search container.
   */
  function createSimilarityContainer($wrapper) {
    var $similarityContainer = $('<div class="similarity-search-container"></div>');
    $wrapper.append($similarityContainer);
    return $similarityContainer;
  }

  /**
   * Behavior for realtime text preview.
   */
  Drupal.behaviors.realtimeTextPreview = {
    attach: function (context, settings) {
      // Handle regular text inputs and textareas
      once('realtime-text-preview', 'input[type="text"], textarea:not(.ckeditor-enabled):not(.ck-editor__editable)', context).forEach(function (element) {
        // Create a preview element.
        var $input = $(element);
        var $wrapper = $('<div class="realtime-preview-wrapper"></div>');
        
        var $similarityContainer = createSimilarityContainer($wrapper);
        
        // Insert the preview after the input.
        $input.after($wrapper);
        
        // Create debounced search function
        const debouncedSearch = debounce(function(text) {
          if (text.trim().length >= 10) {
            performSimilaritySearch(text, $similarityContainer);
          } else {
            $similarityContainer.empty();
          }
        }, 1500); // 1.5 second delay
        
        // Update the preview text when the input changes.
        $input.on('input', function() {
          const inputValue = $input.val();
          // Perform debounced similarity search
          debouncedSearch(inputValue);
        });
      });
      
      // Handle CKEditor 4 instances (for Drupal 8/9)
      if (window.CKEDITOR) {
        Object.keys(CKEDITOR.instances).forEach(function (instanceName) {
          var editor = CKEDITOR.instances[instanceName];
          var editorElement = editor.element.$;
          
          // Skip if already processed
          if ($(editorElement).hasClass('realtime-preview-processed')) {
            return;
          }
          
          // Mark as processed
          $(editorElement).addClass('realtime-preview-processed');
          
          // Create preview container for this editor
          var $wrapper = $('<div class="realtime-preview-wrapper wysiwyg-preview"></div>');
          var $similarityContainer = createSimilarityContainer($wrapper);
          
          $(editor.element.$).closest('.form-item').append($wrapper);
          
          // Create debounced search function for CKEditor 4
          const debouncedSearchCK4 = debounce(function(text) {
            // Strip HTML tags for similarity search
            const plainText = $('<div>').html(text).text();
            if (plainText.trim().length >= 10) {
              performSimilaritySearch(plainText, $similarityContainer);
            } else {
              $similarityContainer.empty();
            }
          }, 1500);
          
          // Update on change
          editor.on('change', function() {
            const editorData = editor.getData();
            debouncedSearchCK4(editorData);
          });
        });
      }
      
      // Handle CKEditor 5 (for Drupal 10)
      // Wait a bit to ensure CKEditor 5 is fully initialized
      setTimeout(function() {
        // Find all CKEditor 5 elements (they have class .ck-editor__editable)
        once('realtime-text-preview-ck5', '.ck-editor__editable', context).forEach(function(element) {
          var $editable = $(element);
          var $editorContainer = $editable.closest('.ck-editor');
          
          // Create preview container for this editor
          var $wrapper = $('<div class="realtime-preview-wrapper wysiwyg-preview"></div>');
          var $similarityContainer = createSimilarityContainer($wrapper);
          
          $editorContainer.after($wrapper);
          
          // Try to find the associated textarea with CKEditor 5
          var $textarea = $editorContainer.siblings('textarea.ck-editor__editable');
          
          // Create debounced search function for CKEditor 5
          const debouncedSearchCK5 = debounce(function(content) {
            // Strip HTML tags for similarity search
            const plainText = $('<div>').html(content).text();
            if (plainText.trim().length >= 10) {
              performSimilaritySearch(plainText, $similarityContainer);
            } else {
              $similarityContainer.empty();
            }
          }, 1500);
          
          // Set up a mutation observer to watch for changes in the editable area
          var observer = new MutationObserver(function(mutations) {
            let content;
            if ($textarea.length) {
              // If we found the textarea, use its value
              content = $textarea.val();
            } else {
              // Fallback to the HTML content of the editable area
              content = $editable.html();
            }
            
            // Perform debounced similarity search
            debouncedSearchCK5(content);
          });
          
          // Start observing the editor for changes
          observer.observe(element, {
            childList: true,
            subtree: true,
            characterData: true,
            attributes: false
          });
          
          // Also listen for input events directly
          $editable.on('input', function() {
            let content;
            if ($textarea.length) {
              content = $textarea.val();
            } else {
              content = $editable.html();
            }
            
            // Perform debounced similarity search
            debouncedSearchCK5(content);
          });
        });
      }, 1000);
    }
  };

})(jQuery, Drupal, once, drupalSettings);