<?php

namespace Drupal\realtime_text_preview\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\realtime_text_preview\Service\SimilaritySearchService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for handling similarity search AJAX requests.
 */
class SimilaritySearchController extends ControllerBase {

  /**
   * The similarity search service.
   *
   * @var \Drupal\realtime_text_preview\Service\SimilaritySearchService
   */
  protected $similaritySearchService;

  /**
   * Constructs a SimilaritySearchController object.
   *
   * @param \Drupal\realtime_text_preview\Service\SimilaritySearchService $similarity_search_service
   *   The similarity search service.
   */
  public function __construct(SimilaritySearchService $similarity_search_service) {
    $this->similaritySearchService = $similarity_search_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('realtime_text_preview.similarity_search')
    );
  }

  /**
   * Handles AJAX requests for similarity search.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response containing search results.
   */
  public function search(Request $request) {
    $text = $request->request->get('text', '');
    $limit = (int) $request->request->get('limit', 5);

    // Minimum text length check to avoid unnecessary API calls.
    if (strlen(trim($text)) < 10) {
      return new JsonResponse([
        'results' => [],
        'message' => 'Text too short for similarity search.',
      ]);
    }

    try {
      // Perform similarity search.
      $results = $this->similaritySearchService->searchSimilarContent($text, $limit);

      // Log the response (only essential parts to avoid large logs).
    \Drupal::logger('milvus_search')->info('Search Results. Response: <pre>@response</pre>', [
        '@response' => print_r($results, TRUE),
    ]); 
      
      // Format results for display.
      $formatted_results = $this->similaritySearchService->formatSearchResults($results);

       // Log the response (only essential parts to avoid large logs).
    \Drupal::logger('milvus_search')->info('Search Formatted Results. Response: <pre>@response</pre>', [
        '@response' => print_r($formatted_results, TRUE),
    ]); 

      return new JsonResponse([
        'results' => $formatted_results,
        'message' => empty($formatted_results) ? 'No similar content found.' : '',
      ]);
    }
    catch (\Exception $e) {
      return new JsonResponse([
        'results' => [],
        'message' => 'Error performing similarity search.',
        'error' => $e->getMessage(),
      ], 500);
    }
  }
}