<?php

namespace Drupal\realtime_text_preview\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configuration form for Realtime Text Preview settings.
 */
class RealtimeTextPreviewSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['realtime_text_preview.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'realtime_text_preview_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('realtime_text_preview.settings');

    $form['similarity_search'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Similarity Search Settings'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    ];

    $form['similarity_search']['enable_similarity_search'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable similarity search'),
      '#description' => $this->t('Enable AI-powered similarity search functionality.'),
      '#default_value' => $config->get('enable_similarity_search') ?? TRUE,
    ];

    $form['similarity_search']['debounce_delay'] = [
      '#type' => 'number',
      '#title' => $this->t('Debounce delay (milliseconds)'),
      '#description' => $this->t('Delay before triggering similarity search after user stops typing.'),
      '#default_value' => $config->get('debounce_delay') ?? 1500,
      '#min' => 500,
      '#max' => 5000,
      '#step' => 100,
      '#states' => [
        'visible' => [
          ':input[name="enable_similarity_search"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['similarity_search']['min_text_length'] = [
      '#type' => 'number',
      '#title' => $this->t('Minimum text length'),
      '#description' => $this->t('Minimum number of characters required before triggering similarity search.'),
      '#default_value' => $config->get('min_text_length') ?? 10,
      '#min' => 5,
      '#max' => 100,
      '#states' => [
        'visible' => [
          ':input[name="enable_similarity_search"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['similarity_search']['max_results'] = [
      '#type' => 'number',
      '#title' => $this->t('Maximum results'),
      '#description' => $this->t('Maximum number of similar content items to display.'),
      '#default_value' => $config->get('max_results') ?? 5,
      '#min' => 1,
      '#max' => 20,
      '#states' => [
        'visible' => [
          ':input[name="enable_similarity_search"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['similarity_search']['similarity_threshold'] = [
      '#type' => 'number',
      '#title' => $this->t('Similarity threshold (%)'),
      '#description' => $this->t('Minimum similarity percentage to display results.'),
      '#default_value' => $config->get('similarity_threshold') ?? 70,
      '#min' => 1,
      '#max' => 100,
      '#step' => 1,
      '#states' => [
        'visible' => [
          ':input[name="enable_similarity_search"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $form['similarity_search']['content_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Content types to search'),
      '#description' => $this->t('Select which content types should be included in similarity search.'),
      '#options' => $this->getContentTypeOptions(),
      '#default_value' => $config->get('content_types') ?? ['article'],
      '#states' => [
        'visible' => [
          ':input[name="enable_similarity_search"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('realtime_text_preview.settings');
    
    // Save similarity search settings
    $config->set('enable_similarity_search', $form_state->getValue('enable_similarity_search'))
      ->set('debounce_delay', $form_state->getValue('debounce_delay'))
      ->set('min_text_length', $form_state->getValue('min_text_length'))
      ->set('max_results', $form_state->getValue('max_results'))
      ->set('similarity_threshold', $form_state->getValue('similarity_threshold'))
      ->set('content_types', array_filter($form_state->getValue('content_types')))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Get available content type options.
   *
   * @return array
   *   Array of content type options.
   */
  protected function getContentTypeOptions() {
    $options = [];
    $content_types = \Drupal::entityTypeManager()->getStorage('node_type')->loadMultiple();

    foreach ($content_types as $content_type) {
      $options[$content_type->id()] = $content_type->label();
    }

    return $options;
  }
}