<?php

namespace Drupal\realtime_text_preview\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\ai\AiProviderPluginManager;
use Drupal\ai\OperationType\Embeddings\EmbeddingsOutput;
use Drupal\gemini_provider\Plugin\AiProvider\GeminiProvider;

/**
 * Service for similarity search using embeddings and Milvus.
 */
class SimilaritySearchService {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The AI provider plugin manager.
   *
   * @var \Drupal\ai\AiProviderPluginManager
   */
  protected $providerManager;

  /**
   * The Milvus service.
   *
   * @var \Drupal\ai_vdb_provider_milvus\MilvusV2
   */
  protected $milvus;

  /**
   * Constructs a SimilaritySearchService object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\ai\AiProviderPluginManager $provider_manager
   *   The AI provider plugin manager.
   * @param mixed $milvus
   *   The Milvus service.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    LoggerChannelFactoryInterface $logger_factory,
    AiProviderPluginManager $provider_manager,
    $milvus
  ) {
    $this->configFactory = $config_factory;
    $this->loggerFactory = $logger_factory;
    $this->providerManager = $provider_manager;
    $this->milvus = $milvus;
  }

  /**
   * Searches for similar content using embeddings.
   *
   * @param string $text
   *   The text to search for similar content.
   * @param int $limit
   *   The maximum number of results to return.
   *
   * @return array
   *   An array of similar content with similarity scores.
   */
  public function searchSimilarContent($text, $limit = 5) {
    try {
      // Check if similarity search is enabled
      $config = $this->configFactory->get('realtime_text_preview.settings');
      if (!$config->get('enable_similarity_search')) {
        return [];
      }

      // Use configured limit if not specified
      if ($limit === 5) {
        $limit = $config->get('max_results') ?? 5;
      }

      // Get embedding for the input text.
      $embedding_vector = $this->generateEmbedding($text);
      
      if (empty($embedding_vector)) {
        return [];
      }

      // Search for similar content using Milvus.
      $results = $this->searchSimilarArticles($embedding_vector, $limit);
      
      // Filter results by similarity threshold
      $similarity_threshold = $config->get('similarity_threshold') ?? 70;
      $filtered_results = [];
      
      foreach ($results as $result) {
        if (isset($result['distance'])) {
          $similarity = $result['distance'] * 100;
          if ($similarity >= $similarity_threshold) {
            $filtered_results[] = $result;
          }
        }
      }
      
      return $filtered_results;
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('realtime_text_preview')->error('Error in similarity search: @message', [
        '@message' => $e->getMessage(),
      ]);
      return [];
    }
  }

  /**
   * Generates embeddings for the given text.
   *
   * @param string $text
   *   The text to generate embeddings for.
   *
   * @return array
   *   The embedding vector.
   */
  protected function generateEmbedding($text) {
    try {
      // Get embedding provider and model ID from config.
      $config = $this->configFactory->get('ai.settings');
      $provider_id = $config->get('default_providers.embeddings.provider_id');
      $model_id = $config->get('default_providers.embeddings.model_id');

      if (empty($provider_id) || empty($model_id)) {
        $this->loggerFactory->get('realtime_text_preview')->warning('AI provider or model ID not configured.');
        return [];
      }

      // Create provider plugin instance.
      /** @var GeminiProvider $provider */
      $provider = $this->providerManager->createInstance($provider_id);

      // Call the embeddings method to generate embeddings.
      /** @var EmbeddingsOutput $result */
      $result = $provider->embeddings($text, $model_id);

      // Extract data in normalized form.
      return $result->getNormalized();
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('realtime_text_preview')->error('Error generating embedding: @message', [
        '@message' => $e->getMessage(),
      ]);
      return [];
    }
  }

  /**
   * Searches for similar articles using Milvus.
   *
   * @param array $embedding_vector
   *   The embedding vector to search for.
   * @param int $limit
   *   The maximum number of results to return.
   *
   * @return array
   *   An array of matched articles with scores.
   */
  protected function searchSimilarArticles(array $embedding_vector, $limit = 5) {
    try {
      // Read Milvus settings.
      $config = $this->configFactory->get('ai_vdb_provider_milvus.settings');
      $collection_name = 'Article';
      $database_name = 'ArticleDatabase';

      // Set connection settings.
      $this->milvus->setApiKey($config->get('api_key'));
      $this->milvus->setBaseUrl($config->get('server'));
      $this->milvus->setPort((int) $config->get('port') ?: 443);

      // Perform the search.
      $response = $this->milvus->search(
        collection_name: $collection_name,
        vector_input: $embedding_vector,
        output_fields: ['id', 'nid'],
        filters: '',
        limit: $limit,
        offset: 0,
        database_name: $database_name
      );

      return $response['data'] ?? [];
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('realtime_text_preview')->error('Milvus search error: @message', [
        '@message' => $e->getMessage(),
      ]);
      return [];
    }
  }

  /**
   * Formats search results for display.
   *
   * @param array $results
   *   The raw search results from Milvus.
   *
   * @return array
   *   Formatted results with titles and similarity scores.
   */
  public function formatSearchResults(array $results) {
    $formatted_results = [];
    $config = $this->configFactory->get('realtime_text_preview.settings');
    $allowed_content_types = $config->get('content_types') ?? ['article'];
    
    // Keep track of processed node IDs to avoid duplicates
    $processed_nids = [];
    
    foreach ($results as $result) {
      // Use nid (node ID) instead of id (chunk ID)
      if (isset($result['nid'])) {
        $node_id = $result['nid'];
        
        // Skip if we've already processed this node
        if (in_array($node_id, $processed_nids)) {
          continue;
        }
        
        // Load the node to get the title.
        $node = \Drupal::entityTypeManager()->getStorage('node')->load($node_id);
        
        if ($node && in_array($node->bundle(), $allowed_content_types)) {
          $similarity = isset($result['distance']) ? round($result['distance'] * 100, 2) : 0;
          
          $formatted_results[] = [
            'chunk_id' => $result['id'] ?? null, // Keep chunk ID for reference
            'nid' => $node_id,
            'title' => $node->getTitle(),
            'similarity' => $similarity,
            'url' => $node->toUrl()->toString(),
            'type' => $node->bundle(),
            'created' => $node->getCreatedTime(),
          ];
          
          // Mark this node as processed
          $processed_nids[] = $node_id;
        }
      }
    }
    
    return $formatted_results;
  }
}